// SPDX-License-Identifier: MIT
pragma solidity ^0.8.20;

import {IGovernor, Governor} from "@openzeppelin-contracts/governance/Governor.sol";
import {GovernorCountingSimple} from "@openzeppelin-contracts/governance/extensions/GovernorCountingSimple.sol";
import {GovernorVotes} from "@openzeppelin-contracts/governance/extensions/GovernorVotes.sol";
import {GovernorVotesQuorumFraction} from "@openzeppelin-contracts/governance/extensions/GovernorVotesQuorumFraction.sol";
import {GovernorTimelockControl} from "@openzeppelin-contracts/governance/extensions/GovernorTimelockControl.sol";
import {TimelockController} from "@openzeppelin-contracts/governance/TimelockController.sol";
import {IVotes} from "@openzeppelin-contracts/governance/utils/IVotes.sol";
import {IERC165} from "@openzeppelin-contracts/interfaces/IERC165.sol";

contract GhostGovernor is
    Governor,
    GovernorCountingSimple,
    GovernorVotes,
    GovernorVotesQuorumFraction,
    GovernorTimelockControl
{
    uint256 private immutable _innerVotingDelay;
    uint256 private immutable _innerVotingPeriod;

    constructor(
        IVotes _ghst,
        TimelockController _timelock,
        uint256 _fraction,
        uint256 _innerVotingDelayValue,
        uint256 _innerVotingPeriodValue
    )
        Governor("GhostGovernor")
        GovernorVotes(_ghst)
        GovernorVotesQuorumFraction(_fraction)
        GovernorTimelockControl(_timelock)
    {
        _innerVotingDelay = _innerVotingDelayValue;
        _innerVotingPeriod = _innerVotingPeriodValue;
    }

    function votingDelay() public view override returns (uint256) {
        return _innerVotingDelay;
    }

    function votingPeriod() public view override returns (uint256) {
        return _innerVotingPeriod;
    }

    function proposalThreshold() public pure override returns (uint256) {
        return 0;
    }

    function state(uint256 proposalId)
        public
        view
        override(Governor, GovernorTimelockControl)
        returns (ProposalState)
    {
        return super.state(proposalId);
    }

    function proposalNeedsQueuing(uint256 proposalId)
        public
        view
        virtual
        override(Governor, GovernorTimelockControl)
        returns (bool)
    {
        return super.proposalNeedsQueuing(proposalId);
    }

    function _queueOperations(
        uint256 proposalId,
        address[] memory targets,
        uint256[] memory values,
        bytes[] memory calldatas,
        bytes32 descriptionHash
    ) internal override(Governor, GovernorTimelockControl) returns (uint48) {
        return super._queueOperations(
            proposalId,
            targets,
            values,
            calldatas,
            descriptionHash
        );
    }

    function _executeOperations(
        uint256 proposalId,
        address[] memory targets,
        uint256[] memory values,
        bytes[] memory calldatas,
        bytes32 descriptionHash
    ) internal override(Governor, GovernorTimelockControl) {
        super._executeOperations(
            proposalId,
            targets,
            values,
            calldatas,
            descriptionHash
        );
    }

    function _cancel(
        address[] memory targets,
        uint256[] memory values,
        bytes[] memory calldatas,
        bytes32 descriptionHash
    ) internal override(Governor, GovernorTimelockControl) returns (uint256) {
        return super._cancel(targets, values, calldatas, descriptionHash);
    }

    function _executor()
        internal
        view
        override(Governor, GovernorTimelockControl)
        returns (address)
    {
        return super._executor();
    }
}
